/************************************************************************
 *   IRC - Internet Relay Chat, doc/example_module.c
 *   Copyright (C) 2001 Hybrid Development Team
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   $Id: m_svs.c,v 1.6 2003/11/05 18:29:47 jeremy Exp $
 */

/* List of ircd includes from ../include/ */
#include "stdinc.h"
#include "handlers.h"
#include "client.h"
#include "common.h"     /* FALSE bleah */
#include "ircd.h"
#include "irc_string.h"
#include "numeric.h"
#include "fdlist.h"
#include "s_bsd.h"
#include "s_conf.h"
#include "s_log.h"
#include "s_serv.h"
#include "send.h"
#include "msg.h"
#include "parse.h"
#include "modules.h"

#include "s_user.h"  /* send_umode_out() */
#include "channel_mode.h" /* MODE_ADD & MOD_DEL */
#include "hash.h"

static void ms_svshost(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);
static void mo_svshost(struct Client *client_p, struct Client *source_p,
                    int parc, char *parv[]);

static void remove_host(struct Client *, char *);
static void change_host(struct Client *, char *, char *);

/* CHGHOST/SVSHOST/SETHOST */
struct Message svshost_msgtab = {
  "SVSHOST", 0, 0, 0, 0, MFLG_SLOW, 0,
  {m_ignore, m_ignore, ms_svshost, m_ignore}
};
struct Message chghost_msgtab = {
  "CHGHOST", 0, 0, 2, 3, MFLG_SLOW, 0,
  {m_ignore, m_not_oper, m_ignore, mo_svshost}
};
struct Message sethost_msgtab = {
  "SETHOST", 0, 0, 2, 3, MFLG_SLOW, 0,
  {m_ignore, m_not_oper, m_ignore, mo_svshost}
};

#ifndef STATIC_MODULES
/* Here we tell it what to do when the module is loaded */
void
_modinit(void)
{
  /* This will add the commands in test_msgtab (which is above) */
  /* SVSHOST/CHGHOST/SETHOST */
  mod_add_cmd(&svshost_msgtab);
//  mod_add_cmd(&chghost_msgtab);
//  mod_add_cmd(&sethost_msgtab);
}

/* here we tell it what to do when the module is unloaded */
void
_moddeinit(void)
{
  /* This will remove the commands in test_msgtab (which is above) */
  /* SVSHOST/CHGHOST/SETHOST */
  mod_del_cmd(&svshost_msgtab);
//  mod_del_cmd(&chghost_msgtab);
//  mod_del_cmd(&sethost_msgtab);
}

/* When we last modified the file (shown in /modlist), this is usually:
 */
const char *_version = "$Revision: 1.6 $";
#endif

/*
 * mo_svshost
 *      parv[0] = sender prefix
 *      parv[1] = target nickname
 *      parv[2] = new hostname
 */
static void mo_svshost(struct Client *client_p, struct Client *source_p,
                   int parc, char *parv[])
{
  struct Client *target_p;
  char *nick, hostname[HOSTLEN+1];
  int remove = 0;

  nick = parv[1];

  if(parc > 2)
    strlcpy(hostname, parv[2], HOSTLEN);

  if(!isalpha(nick[0]))
    if(nick[0] == '-')
      remove = 1;

  if(remove > 0)
    {
      nick++;
      remove_host(source_p, nick);
    }
  else
    {
      if (parc != 3)
        {
          sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS), me.name, source_p->name, "CHGHOST");
          return;
        }
      change_host(source_p, nick, (hostname)? hostname : NULL);
    }

  return;
}

static void remove_host(struct Client *source_p, char *nick)
{
  struct Client *target_p;

  if ((target_p = find_client(nick)) == NULL)
    {
      sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name, source_p->name, target_p->name);
      return;
    }

  strcpy(target_p->host, target_p->rhost);

  sendto_one(source_p, "Reset %s's Host", target_p->name);
  return;
}

static void change_host(struct Client *source_p, char *nick, char *host)
{
  struct Client *target_p;
  char oldhost[HOSTLEN+1];
  int setflags;

  if ((target_p = find_client(nick)) == NULL)
    {
      sendto_one(source_p, form_str(ERR_NOSUCHNICK), me.name, source_p->name, nick);
      return;
    }

  /* check at this point for a valid hostname, and if its null
     which it shouldn't be at this point */
  if (!valid_hostname(host) || (host == NULL) || (strlen(host) > HOSTLEN))
    {
      sendto_one(source_p, "%s :Invalid Hostname", host);
      return;
    }

  if(!strchr(host, '.') && !strchr(host, ':'))
    {
      sendto_one(source_p, "Hostname's must contain one . or :");
      return;
    }

  /* Preserve the old hostname until we are finished */
  strcpy(oldhost, target_p->host);

  /* Prevent any unnecessary bursts */
  if(!strcmp(oldhost, host))
    {
      sendto_one(source_p, "%s :Hostname Identical", host);
      return;
    }

  if(MyClient(target_p) && IsMasked(target_p))
    {
      setflags = target_p->umodes;
      target_p->umodes &= ~UMODE_IPMASK;
      send_umode_out(target_p, target_p, setflags);
    }

  strcpy(target_p->host, host);
  sendto_realops_flags(UMODE_ALL, L_ALL, "CHGHOST was used on %s (%s -> %s)", nick, oldhost, host);
  ilog(L_NOTICE, "%s changed %s's hostname (%s -> %s)", source_p->name, nick, oldhost, host);
  sendto_server(NULL, NULL, CAP_IPMASK, NOCAPS, ":%s SVSHOST %s %s %s", source_p->name, target_p->name, oldhost, host);

  sendto_one(source_p, "Changed %s's Host (%s -> %s)", nick, oldhost, host);
  return;
}

/*
 * ms_svshost
 *      parv[0] = sender prefix
 *      parv[1] = target nickname
 *      parv[2] = new hostname
 */
static void ms_svshost(struct Client *client_p, struct Client *source_p,
                   int parc, char *parv[])
{
  struct Client *target_p;
  char *nick;
  int setflags;
  nick = parv[1];

  if (parc != 4)
    return;

  sendto_server(client_p, NULL, CAP_IPMASK, NOCAPS, ":%s SVSHOST %s %s %s", parv[0], parv[1], parv[2], parv[3]);

  target_p = find_client(nick);  
  strcpy(target_p->host, parv[3]);

  if(!IsServer(client_p))
    {
      sendto_realops_flags(UMODE_ALL, L_ALL, "CHGHOST was used on %s (%s -> %s)", nick, parv[2], parv[3]);
      ilog(L_NOTICE, "%s changed %s's hostname (%s -> %s)", parv[0], nick, parv[2], parv[3]);
    }
  else
    {
      /* Changing the host on a masked user?
         if so pull +X */
      if(MyClient(target_p) && IsMasked(target_p))
        {
          setflags = target_p->umodes;
          target_p->umodes &= ~UMODE_IPMASK;
          send_umode_out(target_p, target_p, setflags);
        }
      ilog(L_NOTICE, "Caught SVSHOST from server [%s], Possibly a change of UMODE_ISMASK", parv[0]);
    }

  return;
}
